@@ -1,10 +1,12 @@
+from __future__ import annotations
+
 import argparse
 import logging
 import re
 import time
 from contextlib import nullcontext
 from operator import eq, gt, lt
-from typing import Any, Type
+from typing import Any
 from unittest.mock import Mock, call, patch
 
 import freezegun
@@ -53,17 +55,22 @@ class CustomConstructorTwoPlugin(FakePlugin):
 
 
 class TestPlugin:
-    @pytest.mark.parametrize(("pluginclass", "module", "logger"), [
-        (Plugin, "plugin", "streamlink.plugin.plugin"),
-        (FakePlugin, "test_plugin", "tests.test_plugin"),
-        (RenamedPlugin, "baz", "foo.bar.baz"),
-        (CustomConstructorOnePlugin, "test_plugin", "tests.test_plugin"),
-        (CustomConstructorTwoPlugin, "test_plugin", "tests.test_plugin"),
-    ])
-    def test_constructor(self, caplog: pytest.LogCaptureFixture, pluginclass: Type[Plugin], module: str, logger: str):
+    @pytest.mark.parametrize(
+        ("pluginclass", "module", "logger"),
+        [
+            (Plugin, "plugin", "streamlink.plugin.plugin"),
+            (FakePlugin, "test_plugin", "tests.test_plugin"),
+            (RenamedPlugin, "baz", "foo.bar.baz"),
+            (CustomConstructorOnePlugin, "test_plugin", "tests.test_plugin"),
+            (CustomConstructorTwoPlugin, "test_plugin", "tests.test_plugin"),
+        ],
+    )
+    def test_constructor(self, caplog: pytest.LogCaptureFixture, pluginclass: type[Plugin], module: str, logger: str):
         session = Mock()
-        with patch("streamlink.plugin.plugin.Cache") as mock_cache, \
-             patch.object(pluginclass, "load_cookies") as mock_load_cookies:
+        with (
+            patch("streamlink.plugin.plugin.Cache") as mock_cache,
+            patch.object(pluginclass, "load_cookies") as mock_load_cookies,
+        ):
             plugin = pluginclass(session, "http://localhost")
 
         assert not caplog.records
@@ -95,19 +102,23 @@ class TestPlugin:
 class TestPluginMatcher:
     # noinspection PyUnusedLocal
     def test_decorator(self):
+        class MyPlugin:
+            pass
+
         with pytest.raises(TypeError) as cm:
-            @pluginmatcher(re.compile(""))
-            class MyPlugin:
-                pass
+            # noinspection PyTypeChecker
+            pluginmatcher(re.compile(""))(MyPlugin)
+
         assert str(cm.value) == "MyPlugin is not a Plugin"
 
-    # noinspection PyUnusedLocal
     def test_named_duplicate(self):
+        class MyPlugin(FakePlugin):
+            pass
+
+        matcher = pluginmatcher(re.compile("http://foo"), name="foo")
+
         with pytest.raises(ValueError, match=r"^A matcher named 'foo' has already been registered$"):
-            @pluginmatcher(re.compile("http://foo"), name="foo")
-            @pluginmatcher(re.compile("http://foo"), name="foo")
-            class MyPlugin(FakePlugin):
-                pass
+            matcher(matcher(MyPlugin))
 
     def test_no_matchers(self):
         class MyPlugin(FakePlugin):
@@ -229,64 +240,67 @@ class TestPluginArguments:
 
         assert tuple(arg.name for arg in MixedPlugin.arguments) == ("qux", "foo", "bar", "baz")
 
-    @pytest.mark.parametrize(("options", "args", "expected", "raises"), [
-        pytest.param(
-            {"type": "int"},
-            ["--myplugin-foo", "123"],
-            123,
-            nullcontext(),
-            id="int",
-        ),
-        pytest.param(
-            {"type": "float"},
-            ["--myplugin-foo", "123.456"],
-            123.456,
-            nullcontext(),
-            id="float",
-        ),
-        pytest.param(
-            {"type": "bool"},
-            ["--myplugin-foo", "yes"],
-            True,
-            nullcontext(),
-            id="bool",
-        ),
-        pytest.param(
-            {"type": "keyvalue"},
-            ["--myplugin-foo", "key=value"],
-            ("key", "value"),
-            nullcontext(),
-            id="keyvalue",
-        ),
-        pytest.param(
-            {"type": "comma_list_filter", "type_args": (["one", "two", "four"], )},
-            ["--myplugin-foo", "one,two,three,four"],
-            ["one", "two", "four"],
-            nullcontext(),
-            id="comma_list_filter - args",
-        ),
-        pytest.param(
-            {"type": "comma_list_filter", "type_kwargs": {"acceptable": ["one", "two", "four"]}},
-            ["--myplugin-foo", "one,two,three,four"],
-            ["one", "two", "four"],
-            nullcontext(),
-            id="comma_list_filter - kwargs",
-        ),
-        pytest.param(
-            {"type": "hours_minutes_seconds"},
-            ["--myplugin-foo", "1h2m3s"],
-            3723,
-            nullcontext(),
-            id="hours_minutes_seconds",
-        ),
-        pytest.param(
-            {"type": "UNKNOWN"},
-            None,
-            None,
-            pytest.raises(TypeError),
-            id="UNKNOWN",
-        ),
-    ])
+    @pytest.mark.parametrize(
+        ("options", "args", "expected", "raises"),
+        [
+            pytest.param(
+                {"type": "int"},
+                ["--myplugin-foo", "123"],
+                123,
+                nullcontext(),
+                id="int",
+            ),
+            pytest.param(
+                {"type": "float"},
+                ["--myplugin-foo", "123.456"],
+                123.456,
+                nullcontext(),
+                id="float",
+            ),
+            pytest.param(
+                {"type": "bool"},
+                ["--myplugin-foo", "yes"],
+                True,
+                nullcontext(),
+                id="bool",
+            ),
+            pytest.param(
+                {"type": "keyvalue"},
+                ["--myplugin-foo", "key=value"],
+                ("key", "value"),
+                nullcontext(),
+                id="keyvalue",
+            ),
+            pytest.param(
+                {"type": "comma_list_filter", "type_args": (["one", "two", "four"],)},
+                ["--myplugin-foo", "one,two,three,four"],
+                ["one", "two", "four"],
+                nullcontext(),
+                id="comma_list_filter - args",
+            ),
+            pytest.param(
+                {"type": "comma_list_filter", "type_kwargs": {"acceptable": ["one", "two", "four"]}},
+                ["--myplugin-foo", "one,two,three,four"],
+                ["one", "two", "four"],
+                nullcontext(),
+                id="comma_list_filter - kwargs",
+            ),
+            pytest.param(
+                {"type": "hours_minutes_seconds"},
+                ["--myplugin-foo", "1h2m3s"],
+                3723,
+                nullcontext(),
+                id="hours_minutes_seconds",
+            ),
+            pytest.param(
+                {"type": "UNKNOWN"},
+                None,
+                None,
+                pytest.raises(TypeError),
+                id="UNKNOWN",
+            ),
+        ],
+    )
     def test_type_argument_map(self, options: dict, args: list, expected: Any, raises: nullcontext):
         class MyPlugin(FakePlugin):
             pass
@@ -309,6 +323,7 @@ class TestPluginArguments:
                 @pluginargument("foo")
                 class Foo:
                     pass
+
         assert str(cm.value) == "Foo is not a Plugin"
 
     def test_empty(self):
@@ -372,12 +387,12 @@ class TestCookies:
             yield cache
 
     @pytest.fixture()
-    def logger(self, pluginclass: Type[Plugin]):
+    def logger(self, pluginclass: type[Plugin]):
         with patch("streamlink.plugin.plugin.logging") as mock_logging:
             yield mock_logging.getLogger(pluginclass.__module__)
 
     @pytest.fixture()
-    def plugin(self, pluginclass: Type[Plugin], session: Streamlink, plugincache: Mock, logger: Mock):
+    def plugin(self, pluginclass: type[Plugin], session: Streamlink, plugincache: Mock, logger: Mock):
         plugin = pluginclass(session, "http://test.se")
         assert plugin.cache is plugincache
         assert plugin.logger is logger
@@ -394,11 +409,13 @@ class TestCookies:
 
     @pytest.mark.parametrize(
         "plugincache",
-        [{
-            "__cookie:test-name1:test.se:80:/": _create_cookie_dict("test-name1", "test-value1"),
-            "__cookie:test-name2:test.se:80:/": _create_cookie_dict("test-name2", "test-value2"),
-            "unrelated": "data",
-        }],
+        [
+            {
+                "__cookie:test-name1:test.se:80:/": _create_cookie_dict("test-name1", "test-value1"),
+                "__cookie:test-name2:test.se:80:/": _create_cookie_dict("test-name2", "test-value2"),
+                "unrelated": "data",
+            },
+        ],
         indirect=True,
     )
     def test_load(self, session: Streamlink, plugin: Plugin, plugincache: Mock, logger: Mock):
@@ -416,11 +433,13 @@ class TestCookies:
         session.http.cookies.set_cookie(cookie2)
 
         plugin.save_cookies(lambda cookie: cookie.name == "test-name1", default_expires=3600)
-        assert plugincache.set.call_args_list == [call(
-            "__cookie:test-name1:test.se:80:/",
-            self.create_cookie_dict("test-name1", "test-value1", None),
-            3600,
-        )]
+        assert plugincache.set.call_args_list == [
+            call(
+                "__cookie:test-name1:test.se:80:/",
+                self.create_cookie_dict("test-name1", "test-value1", None),
+                3600,
+            ),
+        ]
         assert logger.debug.call_args_list == [call("Saved cookies: test-name1")]
 
     @freezegun.freeze_time("1970-01-01T00:00:00Z")
@@ -436,19 +455,23 @@ class TestCookies:
         session.http.cookies.set_cookie(cookie)
 
         plugin.save_cookies(default_expires=60)
-        assert plugincache.set.call_args_list == [call(
-            "__cookie:test-name:test.se:80:/",
-            self.create_cookie_dict("test-name", "test-value", 3600),
-            3600,
-        )]
+        assert plugincache.set.call_args_list == [
+            call(
+                "__cookie:test-name:test.se:80:/",
+                self.create_cookie_dict("test-name", "test-value", 3600),
+                3600,
+            ),
+        ]
 
     @pytest.mark.parametrize(
         "plugincache",
-        [{
-            "__cookie:test-name1:test.se:80:/": _create_cookie_dict("test-name1", "test-value1", None),
-            "__cookie:test-name2:test.se:80:/": _create_cookie_dict("test-name2", "test-value2", None),
-            "unrelated": "data",
-        }],
+        [
+            {
+                "__cookie:test-name1:test.se:80:/": _create_cookie_dict("test-name1", "test-value1", None),
+                "__cookie:test-name2:test.se:80:/": _create_cookie_dict("test-name2", "test-value2", None),
+                "unrelated": "data",
+            },
+        ],
         indirect=True,
     )
     def test_clear(self, session: Streamlink, plugin: Plugin, plugincache: Mock):
@@ -461,11 +484,13 @@ class TestCookies:
 
     @pytest.mark.parametrize(
         "plugincache",
-        [{
-            "__cookie:test-name1:test.se:80:/": _create_cookie_dict("test-name1", "test-value1", None),
-            "__cookie:test-name2:test.se:80:/": _create_cookie_dict("test-name2", "test-value2", None),
-            "unrelated": "data",
-        }],
+        [
+            {
+                "__cookie:test-name1:test.se:80:/": _create_cookie_dict("test-name1", "test-value1", None),
+                "__cookie:test-name2:test.se:80:/": _create_cookie_dict("test-name2", "test-value2", None),
+                "unrelated": "data",
+            },
+        ],
         indirect=True,
     )
     def test_clear_filter(self, session: Streamlink, plugin: Plugin, plugincache: Mock):
@@ -477,56 +502,65 @@ class TestCookies:
         assert tuple(session.http.cookies.keys()) == ("test-name1",)
 
 
-@pytest.mark.parametrize(("params", "expected"), [
-    (
-        None,
-        {},
-    ),
-    (
-        "foo=bar",
-        dict(foo="bar"),
-    ),
-    (
-        "verify=False",
-        dict(verify=False),
-    ),
-    (
-        "timeout=123.45",
-        dict(timeout=123.45),
-    ),
-    (
-        "verify=False params={'key': 'a value'}",
-        dict(verify=False, params=dict(key="a value")),
-    ),
-    (
-        "\"conn=['B:1', 'S:authMe', 'O:1', 'NN:code:1.23', 'NS:flag:ok', 'O:0']",
-        dict(conn=["B:1", "S:authMe", "O:1", "NN:code:1.23", "NS:flag:ok", "O:0"]),
-    ),
-])
+@pytest.mark.parametrize(
+    ("params", "expected"),
+    [
+        (
+            None,
+            {},
+        ),
+        (
+            "foo=bar",
+            dict(foo="bar"),
+        ),
+        (
+            "verify=False",
+            dict(verify=False),
+        ),
+        (
+            "timeout=123.45",
+            dict(timeout=123.45),
+        ),
+        (
+            "verify=False params={'key': 'a value'}",
+            dict(verify=False, params=dict(key="a value")),
+        ),
+        (
+            "\"conn=['B:1', 'S:authMe', 'O:1', 'NN:code:1.23', 'NS:flag:ok', 'O:0']",
+            dict(conn=["B:1", "S:authMe", "O:1", "NN:code:1.23", "NS:flag:ok", "O:0"]),
+        ),
+    ],
+)
 def test_parse_params(params, expected):
     assert parse_params(params) == expected
 
 
-@pytest.mark.parametrize(("weight", "expected"), [
-    ("720p", (720, "pixels")),
-    ("720p+", (721, "pixels")),
-    ("720p60", (780, "pixels")),
-])
+@pytest.mark.parametrize(
+    ("weight", "expected"),
+    [
+        ("720p", (720, "pixels")),
+        ("720p+", (721, "pixels")),
+        ("720p60", (780, "pixels")),
+    ],
+)
 def test_stream_weight_value(weight, expected):
     assert stream_weight(weight) == expected
 
 
-@pytest.mark.parametrize(("weight_a", "operator", "weight_b"), [
-    ("720p+", gt, "720p"),
-    ("720p_3000k", gt, "720p_2500k"),
-    ("720p60_3000k", gt, "720p_3000k"),
-    ("3000k", gt, "2500k"),
-    ("720p", eq, "720p"),
-    ("720p_3000k", lt, "720p+_3000k"),
-    # with audio
-    ("720p+a256k", gt, "720p+a128k"),
-    ("720p+a256k", gt, "360p+a256k"),
-    ("720p+a128k", gt, "360p+a256k"),
-])
+@pytest.mark.parametrize(
+    ("weight_a", "operator", "weight_b"),
+    [
+        ("720p+", gt, "720p"),
+        ("720p_3000k", gt, "720p_2500k"),
+        ("720p60_3000k", gt, "720p_3000k"),
+        ("3000k", gt, "2500k"),
+        ("720p", eq, "720p"),
+        ("720p_3000k", lt, "720p+_3000k"),
+        # with audio
+        ("720p+a256k", gt, "720p+a128k"),
+        ("720p+a256k", gt, "360p+a256k"),
+        ("720p+a128k", gt, "360p+a256k"),
+    ],
+)
 def test_stream_weight(weight_a, weight_b, operator):
     assert operator(stream_weight(weight_a), stream_weight(weight_b))
